#week11:bigram & concordance & word embedding
#read in latest data
load("sampledemo.Rdata")

#method1: 分成相鄰2個字的詞組
library(dplyr)
library(tidytext)
b <- d %>%
  unnest_tokens(bigram, content, token = "ngrams", n = 2)

#計算頻率,可用來尋找新詞 （應在分析前的清理階段進行）
b1=b %>%
  count(bigram, sort = TRUE)
head(b1)

#method 2: 斷詞
library(jiebaR)
cc=worker(byline=T,stop_word = "stop-mac.txt",user = "coviduser-mac.txt")
sd=segment(d$content,cc)

#find top bigrams(=collocation) with quanteda
library(quanteda)
itoken=as.tokens(sd)
library(quanteda.textstats)
bigram=textstat_collocations(itoken,size=2,min_count=10)
head(bigram)
top_n(bigram,10,lambda)

#application 1: discover compound
#make compound collocations (automatically recognize compound words)
bigram1=tokens_compound(itoken,pattern=bigram)
head(bigram1)

#application 2：用規則法測量概念
library(tidyr)
s1 <- bigram%>%
  separate(collocation, c("word1", "word2"), sep = " ")

# example: find top citation sources
source=c("說","表示","指出","提到","稱","認為","強調","聲明")
s2=s1 %>%
  filter(word2 %in% source)%>%
  group_by(word1)%>%
  arrange(desc(count))%>%
  ungroup
s3=s2%>%
  group_by(word1)%>%
  summarise(total=sum(count))%>%
  arrange(desc(total))
head(s3)

#application 3: examine attributes (windows=2)
s1%>%
  filter(word1=="疫苗")%>%
  arrange(desc(count))%>%
  head

#calculate word correlations within particular documents
sd1=d%>%
  unnest_tokens(word, content, token=function(x)segment(x,cc))

#calculate pairwise correlation
#install.packages("widyr")
library(widyr)
word_cors <- sd1 %>%
  group_by(word) %>%
  filter(n() >= 1000) %>%
  pairwise_cor(word,id, sort = TRUE)
head(word_cors)

#find words most correlated with "疫苗"
word_cors %>%
  filter(item1 == "疫苗")%>%
  head

#create a feature co-occurrence matrix(fcm) to specify a window
fcm=fcm(itoken,context="window",size=7)
fcm
topfeatures(fcm["疫苗", ], 20)

#calculate word vectors and find the closest word vectors of key terms
#have a  term-co-occurence matrix(TCM) matrix
library(text2vec)
it <- itoken(sd)
vocab = create_vocabulary(it)
pruned_vocab <- prune_vocabulary(vocab,term_count_min = 10,
                                 doc_proportion_max = 0.5,
                                 doc_proportion_min = 0.001)
vectorizer = vocab_vectorizer(pruned_vocab)
# use window of 5 (tcm=fcm)
tcm = create_tcm(it, vectorizer, skip_grams_window = 5L)
tcm

#factorize it via the GloVe algorithm by fitting the word embedding model
glove = GlobalVectors$new(rank = 50, x_max = 10)

#Extract two sets of word vectors (main and context):get the wordvector matrix
wv_main = glove$fit_transform(tcm, n_iter = 10, convergence_tol = 0.01, n_threads = 8)
wv_context <- glove$components
word_vectors <- wv_main + t(wv_context)

#Examine term representations: find closest word vectors
vac = word_vectors["疫苗", , drop = FALSE] 
cos_sim = sim2(x = word_vectors, y =vac, method = "cosine", norm = "l2") 
head(sort(cos_sim[,1], decreasing = TRUE), 10)

#application 4: make semantic networks
#install.packages("igraph")
library(igraph)
#use bigram counts
bigram_graph=s1%>%
  filter(count>2000)%>%
  graph_from_data_frame()
bigram_graph

#install.packages("ggraph")
library(ggraph)
set.seed(2022)
ggraph(bigram_graph, layout = "fr") +
  geom_edge_link() +
  geom_node_point() +
  geom_node_text(aes(label = name), vjust = 1, hjust = 1,family="STHeitiTC-Medium")

#use phi
word_cors %>%
  filter(correlation > .7) %>%
  graph_from_data_frame() %>%
  ggraph(layout = "fr") +
  geom_edge_link(aes(edge_alpha = correlation), show.legend = FALSE) +
  geom_node_point(color = "lightblue", size = 5) +
  geom_node_text(aes(label = name), repel = TRUE,family="STHeitiTC-Medium") +
  theme_void()

#use quanteda.textplots
#install.packages("quanteda.textplots")
library(quanteda.textplots)
feat <- names(topfeatures(fcm, 50))
fcm_feat=fcm_select(fcm,pattern=feat)
fcm_feat%>%
  textplot_network(min_freq = 0.5, 
                   omit_isolated = T,
                   #vertex_labelfont="Heiti SC Light",
                   color = RColorBrewer::brewer.pal(8,"Dark2"))


#graph 關鍵詞的共現詞
f1 <- names(topfeatures(fcm["疫苗", ], 50))
f2=fcm_select(fcm,pattern=f1)
f2 %>%
  textplot_network(min_freq = 0.5,
                   omit_isolated = T,
                   #vertex_labelfont="Kaiti TC",
                   color = RColorBrewer::brewer.pal(8,"Dark2"))

#prepare edge list for PTT data
dr=read.csv("ptt_gaoduan_articleReviews.csv")
ds=select(dr,artPoster,cmtPoster)
#caculate freq of interaciton
ds1=ds%>%
  group_by(artPoster)%>%
  count(cmtPoster)%>%
  ungroup
mean(ds1$n)
table(ds1$n)

# graph the network among most freq interacted people
net=ds1%>%
  filter(n>10) %>%
  graph_from_data_frame(directed=T)
net
ggraph(net, layout = "fr") +
  geom_edge_link() +
  geom_node_point() +
  geom_node_text(aes(label = name), vjust = 1, hjust = 1,family="STHeitiTC-Medium")
edge_density(net)#%of present edges from all possible edges
reciprocity(net)#% of reciprocated ties
diameter(net)#longest geodesic distance
