load("sampledemo.Rdata")
#斷詞
library(jiebaR)
cc=worker(byline=T,stop_word = "stop-mac.txt",user = "coviduser-mac.txt")
sd=segment(d$content,cc)

#find top coocurred words with quanteda（window=7
library(quanteda)
itoken=as.tokens(sd)
fcm=fcm(itoken,context="window",size=7)
feat <- names(topfeatures(fcm, 30))
fcm_feat=fcm_select(fcm,pattern=feat)

#變成社會網絡資料格式
library(igraph)
g=graph_from_adjacency_matrix(fcm_feat, weighted = T, diag = F)
g
plot(g,vertex.label.family="STKaiti",vertex.size=7,vertex.label.dist=1,edge.label=d$n,edge.width=1,edge.arrow.size=0)

#data conversion
e1=as_data_frame(g, what="edges")
g1=graph_from_data_frame(e1)

#data conversion： xlsx(two mode)--> one mode
library(readxl)
example=read_excel("example.xlsx")
m1=as.matrix(example)
g2=graph_from_incidence_matrix(m1)
plot(g2,vertex.label.family="STKaiti",vertex.size=7,vertex.label.dist=1,edge.label=d$n,edge.width=1,edge.arrow.size=0)
g3=bipartite.projection(m1)
plot(g3$proj1,vertex.label.family="STKaiti",vertex.size=7,vertex.label.dist=1,edge.label=d$n,edge.width=1,edge.arrow.size=0)
plot(g3$proj2,vertex.label.family="STKaiti",vertex.size=7,vertex.label.dist=1,edge.label=d$n,edge.width=1,edge.arrow.size=0)

##global level analysis (descrive the network)
#caculate density: % of present edges from all possible edges
#代表連結程度（陳士榮）
edge_density(g)

#所有邊的平均距離
#代表詞彙間的連結需要的距離（陳士榮）
mean_distance(g, directed=F)

#caculate reciprocity: % of reciprocated links of all links
#表示網絡次級成分的不可及程度，有較多獨立議題（陳士榮）
reciprocity(g)

#caculate global transitivity（网络聚类系数）: ratio of triangles to connected triple
#表示一个图形中节点聚集程度的系数，值越大代表交互关系越大，说明网络越复杂，越能聚类）
#代表詞的關聯性高，呈現好的連結（陳士榮）
transitivity(g, type="global")

#diametr（longest  path)
diameter(g, directed=F)
get_diameter(g, directed=F)

##individual level analysis 
#centrality
# calculate degree
degree(g, mode="all") 
#in-degree
degree(g, mode="in")
#out-degree
degree(g, mode="out")

#plot distribution of degree
deg=degree(g, mode="out")
plot(deg)
hist(deg)

#vertex的大小按照degree的比例
plot(g,vertex.label.family="STKaiti",vertex.size=deg,edge.label=d$n,edge.width=1,edge.arrow.size=0)

#找到degree最高的那個點
V(g)$name[degree(g)==max(degree(g))]

#closeness(與其他點的距離)
closeness(g, mode="all") 

#betweenness(經過他的點的數量)
betweenness(g, directed=T)

#Eigenvector (centrality proportional to the sum of connection centralities)
#eigen_centrality(g, directed=T, weights=NA)
alpha_centrality(g)
alpha_centrality(g,alpha=0.5)

#結構洞
constraint(g)
library(influenceR)
ens(g)


##local-level analysis:分群
#read in the web of interaction
dr=read.csv("ptt_gaoduan_articleReviews.csv")
library(dplyr)
d=select(dr,artPoster,cmtPoster)
df=d%>%
  group_by(artPoster)%>%
  count(cmtPoster)%>%
  ungroup
table(df$n)
library(igraph)
g=df%>%
  filter(n>20) %>%
  graph_from_data_frame(directed=T)
E(g) # The edges of the "g" object 
V(g) # The vertices of the "g" object
E(g)$n# Edge attribute "n"

##cliques
#make network undirected,找出所有可能的群
g1<- as.undirected(g, mode= "collapse", edge.attr.comb=list(weight="sum"))
cliques(g1) # list of cliques
cliques(g1,min=3) #find cliques as big as 3
sapply(cliques(g1), length) # clique sizes 
largest_cliques(g1) # cliques with max number of nodes

#k-core
# find the cores
coreness=graph.coreness(g1)
table(coreness) # a summary of the cores

#method1: Newman-Girvan的演算法：根據邊的中介中心性
ceb <- cluster_edge_betweenness(g1) 
plot(ceb,g1,vertex.size=deg)

length(ceb) # number of communities

membership(ceb) # community membership for each node

modularity(ceb) # how modular the graph partitioning is(高值代表組內很密，組間很鬆)

#method 2: 根據最大化modularity的演算法
cfg <- cluster_fast_greedy(g1) 
plot(cfg, g1,vertex.size=deg)
length(cfg)
membership(cfg)
modularity(cfg)

#method 3: dynamic
cw=cluster_walktrap(g1)
length(cw)
membership(cw)
plot(cw,g1,vertex.size=deg)
modularity(cw)

#method4: multi-level optimization of modularity
cl=cluster_louvain(g1)
length(cl)
membership(cl)
plot(cl,g1,vertex.size=deg)
modularity(cl)

#method5: spinglass (cannot work with unconnected graph)
cs=cluster_spinglass(g1)
length(cs)
membership(cs)
plot(cs,g1,vertex.size=deg)
modularity(cs)

#method6:leading eigenvector algorithm
cle=cluster_leading_eigen(g1)
length(cle)
membership(cle)
plot(cle,g1,vertex.size=deg)
modularity(cle)

#method 7: 根據標籤的替換的演算法：
clp <- cluster_label_prop(g1)
plot(clp, g1,vertex.size=deg)
length(clp)
membership(clp)
modularity(clp)

#homophily
assortativity_degree(g) 

##qap correlation是算矩陣之間的相關係數
#所以必須把csv中資料的讀進來，再變成矩陣
library(readxl)
media=read_excel("media.xlsx")
media$...1=NULL
media_matrix=as.matrix(media)

public=read_excel("public.xlsx")
public$...1=NULL
public_matrix=as.matrix(public)

#install.packages("sna")
library(sna)

#caculate qap correlatoin
gcor(public_matrix,media_matrix)

#build a linear model
nl=netlm(public_matrix,media_matrix)
summary(nl)
