#week7: topic modeling

#get data
load("demo-tm.Rdata")

#系統抽樣每10篇挑一篇
library(dplyr)
d= filter(df,row_number() %% 10 == 1)
save(d,file="sampledemo.Rdata")
load("sampledemo.Rdata")

#一、斷詞
library(jiebaR)
cc=worker(byline=T,stop_word = "stop-mac.txt",user = "coviduser-mac.txt")
sd=segment(d$content,cc)
sd[1:10]

#install.packages("text2vec")
library(text2vec)
it <- itoken(sd)

#二、向量化並製作dtm
#創建詞彙表
vocab = create_vocabulary(it)

#修剪詞彙
pruned_vocab <- prune_vocabulary(vocab,term_count_min = 10,
                                 doc_proportion_max = 0.5,
                                 doc_proportion_min = 0.001)

#construct a document-term matrix
vectorizer = vocab_vectorizer(pruned_vocab)
dtm = create_dtm(it, vectorizer)
dtm

# 三、determine：how many topics? (may take a long time)
#ldatuning package with four tests( Minimization: Arun2010、CaoJuan2009 & Maximization: Deveaud2014、Griffiths2004)
install.packages("ldatuning")
library("ldatuning")

topics=c(2,3,5,7,8,10,12,15,20)
result <- FindTopicsNumber(
 dtm,
  topics = topics,
  metrics = c("Griffiths2004", "CaoJuan2009", "Arun2010", "Deveaud2014"),
  method = "Gibbs",
  control = list(seed = 2020),
  mc.cores = 2L,
  verbose = TRUE
)

#graph
FindTopicsNumber_plot(result)

#四、build the lda model with the best k (k=15)
install.packages("stm")
library(stm)
m = stm(dtm,K = 15,max.em.its = 10)

#五、interpret the model results
#1. display words associated with topics 

#display key words
labelTopics(m)
labelTopics(m, topic=10)

#label all topics
plot(m, type="summary", labeltype = "frex",family="Kaiti TC")

#see a particular topic
cloud(m, topic=10,family="Kaiti TC")

#step 2: understand topics through example documents
thoughts=findThoughts(m,texts = d$content,n=2,topics=10)
d[1457,]

#step 3. report topic prevalence(export gamma)
dt=make.dt(m)
mean(dt$Topic1)

#六. estimating metadata/topic relationships
#model1:the relationship between topic prevalence and date
#改日期的格式
#install.packages("lubridate")
library(lubridate)
d$date=date(d$date)
summary(d$date)
d1=d%>%
  count(date)%>%
  plot(n)

#model1:trend of topics
prep <- estimateEffect(1:15 ~ date, stmobj = m, meta = d)
summary(prep,topics=1)
summary(prep,topics=2:15)

##plot trend
d$month=month(d$date)
m = stm(dtm, K = 15, prevalence =~ month , max.em.its = 10,data=d)
prep <- estimateEffect(1:15 ~ month, stmobj = m, meta = d)
plot(prep,"month",method="continuous",topics=10,model=m)

#add a covariate media type (web media=2 vs. traditional media=1)
prep1 <- estimateEffect(1:15 ~ mediatype+date, stmobj = m, meta = d)
summary(prep1,topics=10)

#七、combine gamma with d
d1=left_join(d,dt,by = c("id"="docnum" ))

save(d1,file="covidtopic.Rdata")

#比較不同media對第一個主題的報導量
f1=aov(d1$Topic10~d1$media)
summary(f1)

#posthoc
TukeyHSD(f1)
print(model.tables(f1,"means"))

#calculate word vectors and find the closest word vectors of key terms
#have a  term-co-occurence matrix(TCM) matrix
tcm = create_tcm(it, vectorizer, skip_grams_window = 5L)

#factorize it via the GloVe algorithm 
glove = GlobalVectors$new(rank =50, x_max = 10)

#Extract two sets of word vectors (main and context)
wv_main = glove$fit_transform(tcm, n_iter = 10, convergence_tol = 0.01, n_threads = 8)
wv_context <- glove$components
word_vectors <- wv_main + t(wv_context)

#Examine term representations: find closest word vectors
library(quanteda)
vac = word_vectors["疫苗", , drop = FALSE] 
cos_sim = sim2(x = word_vectors, y =vac, method = "cosine", norm = "l2") 
head(sort(cos_sim[,1], decreasing = TRUE), 5)

#add more constraints
dom = word_vectors["高端", , drop = FALSE] - 
  word_vectors["AZ疫苗", , drop = FALSE] + 
  word_vectors["聯亞", , drop = FALSE]
cos_sim = sim2(x = word_vectors, y =dom, method = "cosine", norm = "l2") 
head(sort(cos_sim[,1], decreasing = TRUE), 5)


