#week 9: machine learning with quanteda.textmodels

#Task1: predict talklength
#step1: read in labeled data and split it into train set and test set
load("sampledemo.Rdata")
library(dplyr)
d1=select(d,content,talklength)
d1$talklength=as.numeric(d1$talklength)

#split
library(caTools)
set.seed(123)
split=sample.split(d1$talklength,SplitRatio=0.7)
train=subset(d1,split==TRUE)
test=subset(d1,split==FALSE)

#step 2: make dfm for both data set
#for train data: 斷詞、轉換dfm、修剪、替換tf-idf
library(jiebaR)
cc=worker(byline=T, stop_word = "stop-mac.txt",user = "covid-user.txt")
sd=segment(train$content,cc)

#install.packages("quanteda")
library(quanteda)
dfm_train=dfm(as.tokens(sd))
dim(dfm_train)
dfm_train=dfm_trim(dfm_train,
                    min_termfreq = 10,termfreq_type="count",
                    min_docfreq = 0.001,max_docfreq = 0.5,
                    docfreq_type = "prop")
dfm_train_tfidf=dfm_tfidf(dfm_train)

#for test data: 斷詞、轉換dfm、修剪、替換tf-idf
sd1=segment(test$content,cc)
dfm_test=dfm(as.tokens(sd1))
dfm_test=dfm_trim(dfm_test,
                  min_termfreq = 10,termfreq_type="count",
                  min_docfreq = 0.001,max_docfreq = 0.5,
                  docfreq_type = "prop")
dfm_test_tfidf=dfm_tfidf(dfm_test)

#step3: fit a svm model with train data
#install.packages("quanteda.textmodels")
library(quanteda.textmodels)
svm_model=textmodel_svm(dfm_train_tfidf,train$talklength)

#step4: predict test data
pred_svm=predict(svm_model,dfm_test_tfidf)

#check model performance
a=data.frame(pred_svm)
test=cbind(a,test)
#install.packages("Metrics")
library(Metrics)
mae(test$talklength,test$pred_svm) #8.2%
cor.test(test$talklength,test$pred_svm)#.08-->09(tfidf)-->.03(trim)

#step6: predict unlabeled data
load("rawcodingdata.Rdata")
sd5=segment(d5$article,cc)
dfm_new5=dfm(as.tokens(sd5))
dfm_new5_tfidf=dfm_tfidf(dfm_new5)
pred_svm5=predict(svm_model,dfm_new5)
a5=data.frame(pred_svm5)
d5=cbind(a5,d5)

#Task2: learn frames (classification)
#step1:讀入labeled data
d1=select(d5,article,後果)
d1=rename(d1,frame_conseq=後果)

#split
set.seed(123)
split=sample.split(d1$frame_conseq,SplitRatio=0.7)

train1=subset(d1,split==TRUE)
test1=subset(d1,split==FALSE)

#step2: make dfm for train and test data
sd2=segment(train1$article,cc)
dfm_train1=dfm(as.tokens(sd2))
dfm_train1=dfm_trim(dfm_train1,
                    min_termfreq = 10,termfreq_type="count",
                    min_docfreq = 0.001,max_docfreq = 0.5,
                    docfreq_type = "prop")
dfm_train1_tfidf=dfm_tfidf(dfm_train1)

sd3=segment(test1$article,cc)
dfm_test1=dfm(as.tokens(sd3))
dfm_test1=dfm_trim(dfm_test1,
                   min_termfreq = 10,termfreq_type="count",
                   min_docfreq = 0.001,max_docfreq = 0.5,
                   docfreq_type = "prop")
dfm_test1_tfidf=dfm_tfidf(dfm_test1)

#step3:train a svm model
svm_model1=textmodel_svm(dfm_train1_tfidf,train1$frame_conseq)

#step4:predict test data
pred1=predict(svm_model1,dfm_test1_tfidf)

#check performance
a2=data.frame(pred1)
test1=cbind(a2,test1)
table(test1$pred1,test1$frame_conseq)
glmnet:::auc(test1$frame_conseq, test1$pred1)#.78-->.74(tfidf)

#repeat the process with other algorithms(lr, nb & wordscores)
#train a glmnet model
lr_model=textmodel_lr(dfm_train1_tfidf,train1$frame_conseq)
summary(lr_model)
coef(lr_model)

#predict test data
pred_lr=predict(lr_model,dfm_test1_tfidf)

#check performance
a3=data.frame(pred_lr)
test1=cbind(a3,test1)
table(test1$pred_lr,test1$frame_conseq)
glmnet:::auc(test1$frame_conseq, test1$pred_lr)#.59-->.60(tfidf)-->.59(trim)

#do a cross-validation
library(glmnet)
lasso <- cv.glmnet(x = dfm_train1_tfidf,
                   y = (train1$frame_conseq == 1),
                   alpha = 1,
                   nfold = 5,
                   family = "binomial")
#lambda.min is the best model
lasso$lambda.min
index_best <- which(lasso$lambda == lasso$lambda.min)
beta <- lasso$glmnet.fit$beta[, index_best]
head(sort(beta, decreasing = TRUE), 20)

#train a naive bayes model
nb_model=textmodel_nb(dfm_train1_tfidf,train1$frame_conseq)
summary(nb_model)
coef(nb_model)
c=data.frame(coef(nb_model))

#predict test data
pred_nb=predict(nb_model,dfm_test1_tfidf,force=T)

#check performance
a3=data.frame(pred_nb)
test1=cbind(a3,test1)
table(test1$pred_nb,test1$frame_conseq)
glmnet:::auc(test1$frame_conseq, test1$pred_nb)#.74-->76(tfidf&trim)

#train a wordscores text model
ws_model=textmodel_wordscores(dfm_train1_tfidf,train1$frame_conseq)
summary(ws_model)
coef(ws_model)
c=data.frame(coef(ws_model))
top_n(c,30)

#predict test data
pred_ws=predict(ws_model,dfm_test1_tfidf)

#check performance
a4=data.frame(pred_ws)
test1=cbind(a4,test1)
glmnet:::auc(test1$frame_conseq, test1$pred_ws)#.84

#apply the best model (ws_model) to predict new data(sampledemo.Rdata)
sd_new=segment(d$content,cc)
dfm_new=dfm(as.tokens(sd_new))
dfm_new_tfidf=dfm_tfidf(dfm_new)
ws_conseq=predict(ws_model,dfm_new_tfidf)
a4=data.frame(ws_conseq)
d2=cbind(a4,d)
mean(d2$ws_conseq)

#try svm model
svm_conseq=predict(svm_model1,dfm_new_tfidf)
a5=data.frame(svm_conseq)
d2=cbind(a5,d2)
table(d2$svm_conseq)

