##week5: sentiment analysis
#使用ntusd正面詞字典
ntuPosEmo=read.table("ntu-pos-mac.txt", header = T,sep="\r",quote = "", stringsAsFactors = F)

#讀入data
load('demo.RData')

#先給文章一個編號
library(dplyr)
d=mutate(d,id=row_number())

#變成wordlist並去處停頓詞
sw=read.table("stop-mac.txt", header = T,sep="\r",quote = "", stringsAsFactors = F)

#斷詞並去處停頓詞
library(tidytext)
t <- d %>%
  unnest_tokens(word, content)%>%
  anti_join(sw,by="word")

#計算正面情感詞的詞頻（情緒分數）
pe=t %>%
  inner_join(ntuPosEmo) %>%
  count(word, sort = TRUE) 

#用圖形展示正面詞頻
library(ggplot2)
pe %>%
  top_n(10) %>%
  ungroup() %>%
  mutate(word = reorder(word, n)) %>%
  ggplot(aes(word, n)) +
  geom_col(show.legend = FALSE) +
  theme(text=element_text(family="Kaiti TC", size=14))+
  labs(y = "positive sentiment",
       x = NULL) +
  coord_flip()

#文字雲
library(RColorBrewer)
library(wordcloud)
pe %>%
  with(wordcloud(word, n, max.words = 50,family="Kaiti TC"))

#接下來是文章層面的分析
pe1=t %>%
  inner_join(ntuPosEmo) %>%
  count(id) 

#將正面情緒分數合併到最初的資料檔
pe1=pe1%>%
  rename("ntupos"=n)
d=full_join(d,pe1)
d[is.na(d)]=0

#進行文章層面變項與情緒分數之間關係的分析
#run correlation(x=ntupos,y=talklength)
cor.test(d$ntupos,d$talklength)

#run t-test(x=mediatype,y=ntupos)
##重新編碼&存成新變項
table(d$media)
d=mutate(d, mediatype=recode(media, "LINE TODAY"="web media", 
                         "TVBS新聞"="traditional media", 
                         "yahoo新聞"="web media", 
                         "Match生活網"="web media", 
                         "Taiwan News Agency"="traditional media", 
                         "自由時報"="traditional media") )
table(d$mediatype)
t.test(ntupos~mediatype,d)

#run anova (x=media,y=ntupos)
m1=aov(d$ntupos~d$media)
summary(m1)
print(model.tables(m1,"means"))
TukeyHSD(m1)

#讀取一個離散情緒sad
load("cliwc.Rdata")
sad=cliwc%>%
  filter(c1=="Sad"|c2=="Sad"|c3=="Sad"|c4=="Sad"|c5=="Sad")%>%
  select(word)

sad1=t%>%
  inner_join(sad)%>%
  count(id)%>%
  rename("sad"=n)
d=full_join(d,sad1)
d[is.na(d)]=0

save(d,file="demo-tm.Rdata")

##曹修源(2017)
#斷句
t1=d%>%
unnest_tokens(input = content, output = "Sentence", token = "regex",
              pattern = "(?<!\\b\\p{L}r)\\。")

t1=t1%>%
  mutate(sid=row_number())

#斷詞
t2=t1%>%
unnest_tokens(word, Sentence)%>%
  anti_join(sw,by="word")

#情緒分析
ntuNegEmo=read.table("ntu-neg-mac.txt", header = T,sep="\r",quote = "", stringsAsFactors = F)

ne=t2 %>%
  inner_join(ntuNegEmo) %>%
  count(sid,sort = TRUE) 

ne=ne%>%
  rename("ntuneg"=n)
t1=full_join(t1,ne)
t1[is.na(t1)]=0

dic=c("國產","疫苗","高端","聯亞","國光")
dic=as_tibble(dic)
vac=t2%>%
  inner_join(dic,by= c("word" = "value"))%>%
  count(sid)%>%
  rename("vac"=n)
t1=full_join(t1,vac)
t1[is.na(t1)]=0
table(t1$vac)

t1=t1%>%
mutate(vacvalue=ntupos*vac-ntuneg*vac)
mean(t1$vacvalue)
