#week6 tf-idf
#load data
load("demo-tm.Rdata")
#d1=d[1:1000,]

#斷詞
library(jiebaR)
library(dplyr)
library(tidytext)

cc=worker(byline=T, stop_word = "stop-mac.txt",user = "covid-user.txt")

sd <- d %>%
  unnest_tokens(word, 
                content, 
                token = function(x) segment(x, cc) )

#計算tf-idf
t=sd%>%
  count(id,word)%>%
  bind_tf_idf(word,id,n) 


t1=sd %>%
  count(media,word)%>%
  bind_tf_idf(word,media, n) 

#按tf-idf排序
t1=t1%>%
  arrange(desc(tf_idf))

library(ggplot2)
#三個階段同一張圖
t1 %>%
  top_n(20) %>%
  ggplot(aes(word, tf_idf, fill = media)) +
  geom_col() +
  labs(x = NULL, y = "tf-idf") +
  theme(text=element_text(family="STHeitiTC-Medium", size=14))+
  coord_flip()

#三個階段各自一張圖
t1 %>%
  group_by(media)%>%
  top_n(20) %>%
  ungroup %>%
  ggplot(aes(word, tf_idf, fill = media)) +
  geom_col() +
  labs(x = NULL, y = "tf-idf") +
  theme(text=element_text(family="STHeitiTC-Medium", size=14))+
  facet_wrap(~media, ncol=2,scales="free")+
  coord_flip()

#為機器學習做準備：將tidyformat轉換為dtm
#將tidyformat轉換為dtm
dtm=sd%>%
  count(id,word)%>%
  cast_dtm(id, word, n)
dtm

#可先根據tf-idf篩選詞，再製作dtm
mean(t$tf_idf, na.rm=T)
sd(t$tf_idf, na.rm=T)

t2=t%>%
  filter(tf_idf>.05)

dtm1=t2%>%
  cast_dtm(id, word, n)
dtm1

#詞的向量化
#install.packages("text2vec")
library(text2vec)

#tokenize texts（斷詞）
sd1=segment(d$content,cc)
it <- itoken(sd1)

#創建詞彙表
vocab = create_vocabulary(it)
vocab

#修剪詞彙
#vocab,term_count_min(最小詞頻)、doc_proportion_max（最高出現文章比例）
pruned_vocab <- prune_vocabulary(vocab,term_count_min = 10,
                                 doc_proportion_max = 0.5,
                                 doc_proportion_min = 0.001)

#construct a document-term matrix
vectorizer = vocab_vectorizer(pruned_vocab)
dtm2= create_dtm(it, vectorizer)
dim(dtm2)

#improve model performance
# define tfidf model
tfidf = TfIdf$new()
# fit model to train data and transform train data with fitted model
dtm_tfidf = fit_transform(dtm2, tfidf)

##郭文平(2019)
#quanteda功能介紹
install.packages("quanteda")
library(quanteda)
d$newtoken=as.tokens(sd1)

#create a feature co-occurrence matrix(fcm)
fcm=fcm(d$newtoken,context="window",count="frequency",window=7)
fcm

#看某個概念的高頻共現詞
f1=names(topfeatures(fcm["疫苗", ], 50))
head(f1,10)

#用textstat_keyness計算共現詞統計上的顯著程度
#保留了“BNT”前面的5個字和後面的5個字，做成dfm
t1=tokens_keep(d$newtoken,pattern=phrase("BNT"), window=5)
dfm1=dfm(t1)
#保留了t1以外所有的其他字，做成dfm
t0=tokens_remove(d$newtoken,pattern=phrase("BNT"), window=5)
dfm0=dfm(t0)

#針對每一個詞，比較它在這兩個dfm中出現的頻率，並提供chi2統計檢定
#install.packages("quanteda.textstats")
library(quanteda.textstats)
key=textstat_keyness(rbind(dfm1,dfm0),seq_len(ndoc(dfm1)))
#按照chi2的值
head(key,50)

#kwic(keyword in context) #默認window=5
kw=kwic(d$newtoken,pattern="疫苗",window=5)
kw[1:10,]

##deal with data lack normal distribution
hist(d$ntupos)

#solution1
d$size=nchar(d$content)
d=d%>%
  mutate(w_ntupos=ntupos/size)
hist(d$w_ntupos)
#solution 2
d$logw_ntupos=log(d$w_ntupos)
hist(d$logw_ntupos)

#solution 3: replace t test (x=mediatype, y=ntupos)
wilcox.test(w_ntupos ~ mediatype, d)

#replace anova(x=media, y=ntupos)
kruskal.test(w_ntupos ~ media, d)

#pairwise post-hoc tests
pairwise.wilcox.test(d$w_ntupos, d$media,
                     p.adjust.method = "BH")

##linear model
m=lm(talklength~mediatype+ntupos, d)
summary(m)
