
##### Goal: Fetch Article 
# 1. Packages: Rvest, Dplyr, StringR
# 2. HTML structure: elements, tags, attributes.
# 3. CSS Selector: 
#     Elements
#     Attributes (class, id, others...)
#     Advance (node/nodes, parents-child, multi-class, exclusion...)
#     Type Transformation (Rvest -> Character)
# 4. String Operation: combination, replace, trim spaces...

# Reference: https://jtr13.github.io/cc19/web-scraping-using-rvest.html

rm(list=ls())
library(tidyverse)
library(rvest)

url = 'https://ent.ltn.com.tw/news/breakingnews/3762479'
res = read_html(url)

title = res %>%
  html_node('h1') %>%
  html_text()
time = res %>%
  html_node('span.time') %>%
  html_text() %>%
  str_replace('\n','') %>%
  trimws()
text = res %>%
  html_nodes('div[data-desc="內容頁"] p:not([class])') %>%
  html_text() %>%
  paste(collapse="")

# practice 1: how about related news?
related = res %>%
  html_node('div[data-desc="相關新聞"]') %>%
  html_nodes('h3') %>%
  html_text()

# how about replies!? (not possible without "rselenium")
response = res %>%
  html_node('span._5mdd') %>%
  html_text()

# practice 2: find all the related links
related_links = res %>%
  html_node('div[data-desc="相關新聞"]') %>%
  html_nodes('a') %>%
  html_attr("href") 
for(link in related_links){
  print(link)
  l_res = read_html(link)
  l_text = l_res %>%
    html_nodes('div[data-desc="內容頁"] p:not([class])') %>% 
    html_text() %>%
    paste(collapse="")
  print(l_text)
  readline()
}


