
##### Goal: Fetch Articles in a Search Result Page
# 1. Sleep (& Headers)
# 2. Try & Catch Error

rm(list=ls())
library(tidyverse)
library(rvest)

url = 'https://search.ltn.com.tw/list?keyword=%E9%A6%99%E6%B8%AF'
  # httr::set_config(httr::user_agent("me@example.com; +https://example.com/info.html"))
  # REF: https://xml2.r-lib.org/reference/read_xml.html
res = read_html(url)
article_urls = res %>% 
  html_nodes('ul[data-desc="列表"] li > a') %>%
  html_attr("href")

df = data.frame('title'=character(), 'text'=character(), 'time'=character(), 'url'=character())
for(a_url in article_urls){
  print(a_url)
  tryCatch({
    a_res = read_html(a_url)
    title = a_res %>%
      html_node('h1') %>%
      html_text()
    time = a_res %>%
      html_node('span.time') %>%
      html_text() %>%
      str_replace('\n','') %>%
      trimws()
    text = a_res %>%
      html_nodes('div[data-desc="內容頁"] p:not([class])') %>% 
      html_text() %>%
      paste(collapse="")
    article = data.frame('title'=title, 'text'=text, 'time'=time, 'url'=a_url)
    df = rbind(df, article)
  }, 
  error = function(e){print(paste0('Error in url: ', a_url))}
  )
  Sys.sleep(runif(1,0.5,1))
}
write_excel_csv(df, file="../LTNResults.csv", ) # write_csv / write.csv (fileEncoding='UTF-8')